// This file may be redistributed and modified only under the terms of
// the GNU General Public License (See COPYING for details).
// Copyright (C) 2005 - 2006 Simon Goodall

package WFUT;


/**
 * This class encodes a string to RFC 2396 compliance.
 */

public class Encoder {
  protected static final String reserved = ";/?:@&=+$,";
  protected static final String mark = "-_.|~*'()";

  /**
   * Encode an 8 byte char to the stringbuffer
   */
  protected static void encodeByte(StringBuffer enc, int c) {
    enc.append("%");
    enc.append(Integer.toHexString(((int)c & 0xF0)>>4));
    enc.append(Integer.toHexString((int)c & 0x0F));
  }

  public static String encode(final String str) {
    StringBuffer enc = new StringBuffer();
    for (int i = 0; i < str.length(); ++i) {
      char c = str.charAt(i);
      if ((c >= 'a' && c <= 'z')
          || (c >= 'A' && c <= 'Z')
          || (c >= '0' && c <= '9')) {
        // Valid alpha numeric character
        enc.append(c);
      } else {
        // Check for special allowed characters
        if (mark.indexOf(c) != -1) {
          enc.append(c);
        } else if (reserved.indexOf(c) != -1)  {
          enc.append(c);
        } else {
          // This encoding from http://czyborra.com/utf
          if (c < 0x80) {
            encodeByte(enc, c);
          }
          else if (c < 0x800) {
            encodeByte(enc, 0xC0 | c >> 6);
            encodeByte(enc, 0x80 | c & 0x3F);
          }
          else if (c < 0x10000) {
            encodeByte(enc, 0xE0 | c >> 18);
            encodeByte(enc, 0x80 | c >> 6 & 0x3F);
            encodeByte(enc, 0x80 | c & 0x3F);
          }
          else if (c < 0x200000) {
            encodeByte (enc, 0xF0 | c >> 18);
            encodeByte(enc, 0xE0 | c >> 18);
            encodeByte(enc, 0x80 | c >> 6 & 0x3F);
            encodeByte(enc, 0x80 | c & 0x3F);
          }
        }
      }
    }
    return enc.toString();
  }
}
